// kleval.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Compute the kernel matrix of a kernel.

#include "kernel/lib/kernel.h"


DEFINE_int64(data_size, -1,
             "size of dataset to be evaluated [-1 means full dataset]");
DEFINE_bool(libsvm, false, "use libsvm precomputed kernel format");
DEFINE_int64(xmin,  0, "restrict to k(x,y) where x >= xmin");
DEFINE_int64(xmax, -1, "restrict to k(x,y) where x < xmax");
DEFINE_int64(ymin,  0, "restrict to k(x,y) where y >= ymin");
DEFINE_int64(ymax, -1, "restrict to k(x,y) where y < ymax");
DEFINE_bool(null, false, "null output");
DEFINE_bool(kar, false,
            "output kar file with cache instead of matrix in ascii");
DEFINE_bool(bin, false, "output kernel matrix in binary format (double)");

using namespace kernel;

int main(int argc, char** argv) {
  string usage = "Compute the kernel matrix.\n\n  Usage: ";
  usage += argv[0];
  usage += " kernel.kar\n";

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc > 2) {
    ShowUsage();
    return 1;
  }
  string in = argc > 1 ? string(argv[1]) : string();

  KernelBase *kernel = KernelBase::Read(in);
  if(!kernel)
    LOG(FATAL) << "kleval: failed to read " << in;

  int64 n = (FLAGS_data_size == -1 || FLAGS_data_size > kernel->Data()->Size())
             ? kernel->Data()->Size()
             : FLAGS_data_size;
  int64 xmax = (FLAGS_xmax == -1 ? n : FLAGS_xmax);
  int64 ymax = (FLAGS_ymax == -1 ? n : FLAGS_ymax);
  for (int64 i = FLAGS_xmin; i < xmax; ++i) {
    for (int64 j = FLAGS_ymin; j < ymax; ++j) {
      if (FLAGS_null || FLAGS_kar) {
        kernel->Eval(i,j);
        continue;
      }
      if (FLAGS_bin) {
        double d = kernel->Eval(i,j);
        WriteType(std::cout, d);
        continue;
      }
      if (j) cout << " ";
      if (FLAGS_libsvm) {
        if (!j) cout << "0:" << i+1 << " ";
        cout << j+1 << ":";
      }
      cout << kernel->Eval(i,j);
    }
    if (!FLAGS_null && !FLAGS_kar && !FLAGS_bin) cout << endl;
  }

  if (FLAGS_kar)
    kernel->Write(string());

  delete kernel;
  return 0;
}
