// klngramfst.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Generate a n-gram counting fst

#include "kernel/lib/ngram-fst.h"

using namespace kernel;
using namespace fst;

typedef VectorFst<LogArc> LogVectorFst;

DEFINE_int32(order, -1, "n-gram order");
DEFINE_int32(sigma, -1, "alphabet size (i.e. max label)");
DEFINE_bool(gappy, false, "counts gappy n-gram");
DEFINE_int32(max_gap, -1, "max number of gaps allowed [default: order - 1]");
DEFINE_double(lambda, 0.1, "cost penalty for each gap");
DEFINE_int32(min_order, -1, "minimum order of n-gram to be counted [default: same as order]");

int main(int argc, char** argv) {
  string usage = "Generate an n-gram counting fst.\n\n  Usage: ";
  usage += argv[0];
  usage += " > out.fst \n";

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc != 1) {
    ShowUsage();
    return 1;
  }

  if (FLAGS_order == -1)
    LOG(FATAL) << "klngram: n-gram order need to be specified "
               << "with --order=n\n";

  if (FLAGS_sigma == -1)
    LOG(FATAL) << "klngram: value of larger symbol need to be specified "
               << "with --sigma=n\n";

  LogVectorFst fst;
  if (FLAGS_gappy)
    GappyNgram(&fst, FLAGS_order, FLAGS_sigma, -log(FLAGS_lambda),
               FLAGS_max_gap == -1 ? FLAGS_order -1 : FLAGS_max_gap,
               FLAGS_min_order);
  else
    Ngram(&fst, FLAGS_order, FLAGS_sigma, FLAGS_min_order);
  ArcSort(&fst, ILabelCompare<LogArc>());
  fst.Write(string());
}
