#include "svm-kernel.h"
#include <fstream>
#include <string>
#include <math.h>

#include "kernel/lib/kernel.h"

using namespace kernel;
using namespace fst;

class OpenKernel : public GenericKernel {
 public:
  typedef KernelBase Kernel;
  
  OpenKernel(string kernelfile)
      : kernelfile_(kernelfile), n_(0) {}

  void Init () {
    kernel_ = Kernel::Read(kernelfile_);
    if (!kernel_->Data())
      LOG(FATAL) << "openkernel: no data set specified for kernel: " 
                 << kernelfile_;
  }
      
  virtual double Compute(const svm_node *px, const svm_node *py)
  {
    n_++;
    double kxy = kernel_->Eval(px->index - 1, py->index - 1);
    return kxy;
  }
  
  virtual void WriteParams(FILE *fd) {
    fprintf(fd, "kernel %s\n", kernelfile_.c_str(), 
            kernelfile_.c_str()); 
  }
  
  virtual const char* Name() {return name_.c_str();}

  virtual ~OpenKernel() {
    cerr << "openkernel: " << n_ << " kernel computations" << endl;
    delete kernel_;
  }
  
 private:
  static const std::string name_;

  Kernel *kernel_;
  string kernelfile_;
  int n_;
};

const std::string OpenKernel::name_ = "openkernel";

static const int32 kKernelMagicNumber = 2129983209;

extern "C" {
  void *openkernel_load(FILE *fd, char *filename = 0) {
    int32 magic = 0;
    OpenKernel *kernel = 0;
    if (filename && !ftell(fd)) {
      fread(&magic, sizeof(int32), 1, fd);
      fseek(fd, 0, SEEK_SET);
    }
    if (magic == kKernelMagicNumber) {
      kernel =  new OpenKernel(filename);
    }
    else {
      char cmd[256];
      char file[256];
      std::cerr << "Loading open kernel" << std::endl;
      fscanf(fd, "%s", cmd);
      if (strcmp(cmd, "kernel"))
      LOG(error) << "openkernel_load: first parameter should be kernel";
      fscanf(fd, "%s", file);
      std::cerr << "open kernel: " << file << endl;
      kernel =  new OpenKernel(file);
    }
    kernel->Init();
    std::cerr << "open kernel successfully loaded" << std::endl;
    return static_cast<void *>(kernel);
  }
}
